@@ -24,9 +24,7 @@ def exists(val):
     return val is not None
 
 def default(val, d):
-    if exists(val):
-        return val
-    return d() if callable(d) else d
+    return val if exists(val) else d
 
 def identity(t, *args, **kwargs):
     return t
@@ -261,65 +259,6 @@ class ParallelTransformerBlock(nn.Module):
 
         return attn_out + ff_out
 
-# cross attention
-
-class CrossAttention(nn.Module):
-    def __init__(
-        self,
-        dim,
-        dim_head = 64,
-        heads = 8,
-        dropout = 0.
-    ):
-        super().__init__()
-        inner_dim = heads * dim_head
-        self.scale = dim_head ** -0.5
-        self.heads = heads
-
-        self.norm = LayerNorm(dim)
-        self.to_q = nn.Linear(dim, inner_dim, bias = False)
-        self.to_out = nn.Linear(inner_dim, dim, bias = False)
-
-        self.dropout = nn.Dropout(dropout)
-
-    def forward(
-        self,
-        x,
-        kv: Tuple[torch.Tensor, torch.Tensor],
-        context_mask = None
-    ):
-        x = self.norm(x)
-
-        # queries
-
-        q, k, v = self.to_q(x), *kv
-
-        # split out heads and scale queries
-
-        q = rearrange(q, 'b n (h d) -> b h n d', h = self.heads)
-        q = q * self.scale
-
-        # similarity
-
-        sim = einsum('b h i d, b j d -> b h i j', q, k)
-
-        # attention
-
-        attn = sim.softmax(dim = -1)
-        attn = self.dropout(attn)
-
-        # aggregate values
-
-        out = einsum('b h i j, b j d -> b h i d', attn, v)
-
-        # merge heads
-
-        out = rearrange(out, 'b h n d -> b n (h d)')
-
-        # combine heads
-
-        return self.to_out(out)
-
 # transformer
 
 @beartype
@@ -339,28 +278,20 @@ class PaLM(nn.Module):
         lora_r = 8,
         rotary_xpos_scale_base = 512,
         finetune_scopes = tuple(),
-        cross_entropy_ignore_index = 0,
-        cross_attend = False,
-        default_start_token_id = None
+        cross_entropy_ignore_index = 0
     ):
         super().__init__()
         self.dim = dim
         self.dim_head = dim_head
         self.heads = heads
         self.causal = causal
-
         self.num_tokens = num_tokens
-        self.default_start_token_id = default_start_token_id
-
-        self.cross_attend = cross_attend
 
         self.token_emb = nn.Embedding(num_tokens, dim)
         self.layers = nn.ModuleList([])
 
-        self.to_cross_attn_key_values = nn.Linear(dim, dim_head * 2 * depth, bias = False)
-
         for _ in range(depth):
-            self_attn_and_ff = Residual(ParallelTransformerBlock(
+            block = Residual(ParallelTransformerBlock(
                 dim = dim,
                 causal = causal,
                 dim_head = dim_head,
@@ -371,19 +302,7 @@ class PaLM(nn.Module):
                 xpos_scale_base = rotary_xpos_scale_base
             ))
 
-            cross_attn = None
-            if cross_attend:
-                cross_attn = Residual(CrossAttention(
-                    dim = dim,
-                    dim_head = dim_head,
-                    heads = heads,
-                    dropout = attn_dropout
-                ))
-
-            self.layers.append(nn.ModuleList([
-                cross_attn,
-                self_attn_and_ff
-            ]))
+            self.layers.append(block)
 
         self.norm = LayerNorm(dim)
         self.to_logits = nn.Linear(dim, num_tokens, bias=False)
@@ -450,7 +369,7 @@ class PaLM(nn.Module):
 
         lora_modules = self.finetune_modules.pop(scope)
 
-        for (_, layer), (lora_q, lora_k, lora_v, lora_o) in zip(self.layers, lora_modules):
+        for layer, (lora_q, lora_k, lora_v, lora_o) in zip(self.layers, lora_modules):
             block = layer.fn
 
             fused_attn_ff_weight = block.fused_attn_ff_proj.weight
@@ -477,16 +396,6 @@ class PaLM(nn.Module):
         assert scope in self.finetune_modules, f'finetune parameters of scope {scope} not found'
         return self.finetune_modules[scope].parameters()
 
-    # default tokens
-
-    def default_token_ids(self, batch = 1):
-        device = self.device
-
-        if exists(self.default_start_token_id):
-            return torch.full((batch, 1), self.default_start_token_id, device = device)
-
-        return torch.randint(0, self.num_tokens, (batch, 1), device = device)
-
     # generate function
 
     @torch.no_grad()
@@ -502,15 +411,11 @@ class PaLM(nn.Module):
         eos_token = None,
         return_seq_without_prompt = True,
         use_tqdm = False,
-        context = None,
-        batch_size = 1,
         **kwargs
     ):
-        assert self.causal
-
         if not exists(prompt):
-            batch_size = context.shape[0] if exists(context) else batch_size
-            prompt = self.default_token_ids(batch = batch_size)
+            prompt = torch.randint(0, self.num_tokens, (1, 1))
+            prompt = prompt.to(self.device)
             return_seq_without_prompt = False
 
         prompt, leading_dims = pack([prompt], '* n')
@@ -521,7 +426,7 @@ class PaLM(nn.Module):
         sample_num_times = max(1, seq_len - prompt.shape[-1])
 
         for _ in wrapper_fn(range(sample_num_times)):
-            logits, embeds = self.forward(out, return_logits_with_embedding = True, context = context, **kwargs)
+            logits, embeds = self.forward(out, return_logits_with_embedding = True, **kwargs)
             logits, embeds = logits[:, -1], embeds[:, -1]
 
             if exists(filter_logits_fn):
@@ -549,10 +454,7 @@ class PaLM(nn.Module):
 
     def forward(
         self,
-        prompt,
-        mask = None,
-        context = None,
-        context_mask = None,
+        x,
         return_loss = False,
         disable_lora = False,
         finetune_scope = None,
@@ -560,13 +462,18 @@ class PaLM(nn.Module):
         return_only_embedding = False,
         return_logits_with_embedding = False
     ):
-        x = default(prompt, lambda: self.default_token_ids(batch = (context.shape[0] if exists(context) else 1)))
-
-        assert not (exists(context) and not self.cross_attend)
-
         if return_loss:
             x, labels = x[:, :-1], x[:, 1:]
 
+        # mask if encoder
+        # treat any token ids that are negative as tokens to mask out - only needed if not autoregressive
+
+        if not self.causal:
+            mask = x >= 0
+            x = x.masked_fill(~mask, 0)
+        else:
+            mask = None
+
         # get token embedding
 
         x = self.token_emb(x)
@@ -581,22 +488,10 @@ class PaLM(nn.Module):
             assert finetune_scope in self.finetune_modules
             finetune_modules = self.finetune_modules[finetune_scope]
 
-        # cross attention key values, if needed, projected all at once across all decoder layers
-
-        cross_attn_key_values = tuple()
-        if exists(context):
-            context = self.to_cross_attn_key_values(context)
-            context = rearrange(context, 'b n (l r d) -> b n l r d', r = 2, l = len(self.layers))
-            cross_attn_key_values = tuple(tensor.unbind(dim = -2) for tensor in context.unbind(dim = -3))
-
         # parallel attention / ff blocks, passing in finetuning loras
 
-        for (cross_attn, self_attn_and_ff), finetune_modules, cross_attn_kv in zip_longest(self.layers, finetune_modules, cross_attn_key_values):
-
-            if exists(cross_attn) and exists(cross_attn_kv):
-                x = cross_attn(x, kv = cross_attn_kv, context_mask = context_mask)
-
-            x = self_attn_and_ff(x, mask = mask, finetune_modules = finetune_modules)
+        for layer, finetune_modules in zip_longest(self.layers, finetune_modules):
+            x = layer(x, mask = mask, finetune_modules = finetune_modules)
 
         # final norm
 
